#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
import html
import docutils.core

# import interfaces
from pyams_utils.interfaces.tales import ITALESExtension
from pyams_utils.interfaces.text import IHTMLRenderer
from pyramid.interfaces import IRequest

# import packages
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.request import check_request
from pyams_utils.vocabulary import vocabulary_config
from zope.interface import Interface
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


def get_text_start(text, length, max=0):
    """Get first words of given text with maximum given length
    
    If *max* is specified, text is shortened only if remaining text is longer this value
    
    :param str text: initial text
    :param integer length: maximum length of resulting text
    :param integer max: if > 0, *text* is shortened only if remaining text is longer than max

    >>> from pyams_utils.text import get_text_start
    >>> get_text_start('This is a long string', 10)
    'This is a&#133;'
    >>> get_text_start('This is a long string', 20)
    'This is a long&#133;'
    >>> get_text_start('This is a long string', 20, 7)
    'This is a long string'
    """
    result = text or ''
    if length > len(result):
        return result
    index = length - 1
    text_length = len(result)
    while (index > 0) and (result[index] != ' '):
        index -= 1
    if (index > 0) and (text_length > index + max):
        return result[:index] + '&#133;'
    return text


@adapter_config(name='raw', context=(str, IRequest), provides=IHTMLRenderer)
class BaseHTMLRenderer(object):
    """Raw text HTML renderer

    This renderer renders input text 'as is', mainly for use in a <pre> tag.
    """

    def __init__(self, context, request):
        self.context = context
        self.request = request

    def render(self, **kwargs):
        return self.context


@adapter_config(name='text', context=(str, IRequest), provides=IHTMLRenderer)
class TextRenderer(BaseHTMLRenderer):
    """Basic text HTML renderer

    This renderer only replace newlines with HTML breaks.
    """

    def render(self, **kwargs):
        return html.escape(self.context).replace('\n', '<br />\n')


@adapter_config(name='rest', context=(str, IRequest), provides=IHTMLRenderer)
class ReStructuredTextRenderer(BaseHTMLRenderer):
    """reStructuredText HTML renderer

    This renderer is using *docutils* to render HTML output.
    """

    def render(self, **kwargs):
        """Render reStructuredText to HTML"""
        overrides = {
            'halt_level': 6,
            'input_encoding': 'unicode',
            'output_encoding': 'unicode',
            'initial_header_level': 3,
        }
        if 'settings' in kwargs:
            overrides.update(kwargs['settings'])
        parts = docutils.core.publish_parts(self.context,
                                            writer_name='html',
                                            settings_overrides=overrides)
        return ''.join((parts['body_pre_docinfo'], parts['docinfo'], parts['body']))


def text_to_html(text, renderer='text'):
    """Convert text to HTML using the given renderer

    Renderer name can be any registered HTML renderer adapter
    """
    request = check_request()
    registry = request.registry
    renderer = registry.queryMultiAdapter((text, request), IHTMLRenderer, name=renderer)
    if renderer is not None:
        return renderer.render()


@adapter_config(name='html', context=(Interface, Interface, Interface), provides=ITALESExtension)
class HTMLTalesExtension(ContextRequestViewAdapter):
    """*extension:html* TALES expression

    If first *context* argument of the renderer is an object for which an :py:class:`IHTMLRenderer`
    adapter can be found, this adapter is used to render the context to HTML; if *context* is a string,
    it is converted to HTML using the renderer defined as second parameter; otherwise, context is just
    converted to string using the :py:func:`str` function.
    """

    def render(self, context=None, renderer='text'):
        if context is None:
            context = self.context
        registry = self.request.registry
        adapter = registry.queryMultiAdapter((context, self.request, self.view), IHTMLRenderer)
        if adapter is None:
            adapter = registry.queryMultiAdapter((context, self.request), IHTMLRenderer)
        if adapter is not None:
            return adapter.render()
        elif isinstance(context, str):
            return text_to_html(context, renderer)
        else:
            return str(context)


@vocabulary_config(name='PyAMS HTML renderers')
class RenderersVocabulary(SimpleVocabulary):
    """Text renderers vocabulary"""

    def __init__(self):
        request = check_request()
        registry = request.registry
        translate = registry.localizer.translate
        terms = [SimpleTerm(name, name, translate(adapt.title).label)
                 for name, adapt in registry.getAdapters(('', request), IHTMLRenderer)]
        super(RenderersVocabulary, self).__init__(terms)
